SystemOrganization addCategory: #CollectionExtensions!

!Set methodsFor: '*CollectionExtensions' stamp: 'stephane.ducasse 10/16/2008 15:51'!
flatCollect: aBlock
	"Evaluate aBlock for each of the receiver's elements and answer the
	union of all resulting values. Assumes that aBlock returns some kind
	of collection for each element."

	| set |
	self isEmpty ifTrue: [^self copy].
	set := Set new.
	self do: [ :each |
		set addAll: (aBlock value: each) ].
	^set! !

!Symbol methodsFor: '*CollectionExtensions' stamp: 'stephane.ducasse 10/14/2008 22:07'!
value
	"Allow this object to act as a ValueHolder on itself."

	^self! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'stephane.ducasse 10/14/2008 22:28'!
collectAsSet: aBlock
	"Evaluates aBlock for each element of the receiver and collects
	the resulting values into a Set."

	"This is an efficient shorthand for [ (self collect: aBlock) asSet ]."
	"originally developed by a. kuhn and released under MIT."
	
	^self inject: Set new into: [ :set :each | 
		set add: (aBlock value: each); yourself ].! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'stephane.ducasse 10/14/2008 23:10'!
flatCollect: aBlock
	"Evaluate aBlock for each of the receiver's elements and answer the
	list of all resulting values flatten one level. Assumes that aBlock returns some kind
	of collection for each element. Equivalent to the lisp's mapcan"
	
	"original written by a. Kuhn and released under MIT"
	
	| stream |
	self isEmpty ifTrue: [^self copy ].
	stream := (self species new: self size) writeStream. 	
	self do: [ :each |
		stream nextPutAll: (aBlock value: each) ].
	^stream contents! !

!Collection methodsFor: '*CollectionExtensions' stamp: 'stephane.ducasse 10/14/2008 22:28'!
groupedBy: aBlock
	"Return a dictionary whose keys are the result of evaluating aBlock for all elements in
	 the collection, and the value for each key is the collection of elements that evaluated
	 to that key. e.g.
	     #(1 2 3 4 5) groupedBy: [:each | each odd]
	   a Dictionary  
	     true ---> #( 1 3 5)
	     false --> #(2 4)
	originally developed by a. kuhn and released under MIT."

	| result |
	result := Dictionary new.
	self do:
		[:each | | key collection |
		key := aBlock value: each.
		collection := result at: key ifAbsentPut: [OrderedCollection new].
		collection add: each].
	self species ~~ OrderedCollection ifTrue:
		["Convert the result collections to be the right type.
		  Note that it should be safe to modify the dictionary
		  while iterating because we only replace values for existing keys"
		result keysAndValuesDo:
			[:key :value | result at: key put: (self species withAll: value)]].

	^result! !

TestCase subclass: #CollectionExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionExtensions'!

!CollectionExtensionTest methodsFor: 'collect as set' stamp: 'stephane.ducasse 10/14/2008 22:15'!
testCollectAsSet
	"self debug: #testCollectAsSet"
	
	self assert: ((#() collectAsSet: [:each | each odd]) = Set new).
	self assert: (#(1 2 3 4 5 6) collectAsSet: [:each | each odd])
					 = (Set with: true with: false).
	self assert: (#(1 3 5 7 9 11) collectAsSet: [:each | each odd]) 
					= (Set with: true).
					
	self assert: (#(1 2 3 4 5 4 3 2 1) collectAsSet: [:each | each]) = (1 to: 5) asSet.
	
	! !

!CollectionExtensionTest methodsFor: 'collect as set' stamp: 'stephane.ducasse 10/14/2008 22:08'!
testCollectAsSetUsingSymbol
	"self debug: #testCollectAsSetUsingSymbol"
	
	self assert: ((#() collectAsSet: #odd) = Set new).
	self assert: (#(1 2 3 4 5 6) collectAsSet: #odd)
					 = (Set with: true with: false).
	self assert: (#(1 3 5 7 9 11) collectAsSet: #odd) 
					= (Set with: true).! !

!CollectionExtensionTest methodsFor: 'flat collect' stamp: 'stephane.ducasse 10/14/2008 23:06'!
testFlatCollect
	"self debug: #testFlatCollect"
	
	! !

!CollectionExtensionTest methodsFor: 'flat collect' stamp: 'stephane.ducasse 10/14/2008 23:27'!
testFlatCollectArray
	"self debug: #testFlatCollectArray"
	
	self assert: ((#((1 2) (3 4) (5 3)) flatCollect: [ :each ]) = #(1 2 3 4 5 3)).
	self assert: ((#((1 2) (2 3) (1 3 4)) flatCollect: [:each]) = #(1 2 2 3 1 3 4)). 	

	self assert: ((#((1 2) (2 3) (1 3 4)) flatCollect: [:each| Set with: each])
					=  #(#(1 2) #(2 3) #(1 3 4))).
					
	
	
! !

!CollectionExtensionTest methodsFor: 'flat collect' stamp: 'stephane.ducasse 10/16/2008 16:00'!
testFlatCollectSet
	"self debug: #testFlatCollectSet"
	
	self assert: ((#((1 2) (1 2) (1 3 4)) asSet  flatCollect: [:each]) = #(1 1 2 3 4) asSet).
	self assert: ((#() asSet flatCollect: [:each]) = #() asSet).

	self assert:  ((#((1 2) () (1 3 4)) asSet  flatCollect: [:each]) = #(1 1 2 3 4) asSet).
	self assert:  ((#((1 2) #((99)) (1 3 4)) asSet  flatCollect: [:each]) 
					= #(1 1 2 3 4 (99)) asSet).
	self assert:  ((#((1 2) #(()) (1 3 4)) asSet  flatCollect: [:each]) 
					= #(1 1 2 3 4 ()) asSet).! !

!CollectionExtensionTest methodsFor: 'groupedBy' stamp: 'stephane.ducasse 10/14/2008 22:25'!
testGroupedBy
	"self debug: #testGroupedBy"
	
	| res |
	res := #(1 2 3 4 5) asOrderedCollection 
				groupedBy: [:each | each odd].
	self assert:   (res at: true) = #(1 3 5) asOrderedCollection.
	self assert: (res at: false) = #(2 4) asOrderedCollection! !

!CollectionExtensionTest methodsFor: 'groupedBy' stamp: 'stephane.ducasse 10/14/2008 22:23'!
testGroupedByArray
	"self debug: #testGroupedByArray"
	
	| res |
	res := #(1 2 3 4 5) groupedBy: [:each | each odd].
	self assert:   (res at: true) = #(1 3 5).
	self assert: (res at: false) = #(2 4)! !

!CollectionExtensionTest methodsFor: 'groupedBy' stamp: 'stephane.ducasse 10/14/2008 22:24'!
testGroupedBySet
	"self debug: #testGroupedBySet"
	
	| res |
	res := #(1 2 3 4 5 3 4 5) asSet groupedBy: [:each | each odd].
	self assert: (res at: true) = #(1 3 5) asSet.
	self assert: (res at: false) = #(2 4) asSet! !

!CollectionExtensionTest methodsFor: 'symbol - value' stamp: 'stephane.ducasse 10/14/2008 22:10'!
testSymbolInPlaceOfBlock
	"self debug: #testSymbolInPlaceOfBlock"
	
	self assert: (#(1 2 3 4) collect: #odd) =  #(true false true false).
	self assert: (#(1 2 3 4) select: #odd) =  #(1 3).! !
